//+------------------------------------------------------------------+
//|                                            CFractalSeriesSet.mqh |
//|                                Copyright 2019, Korotchenko Roman |
//|                        https://login.mql5.com/en/users/Solitonic |
//|                                             Revision 14 May 2019 |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, Korotchenko Roman"
#property link      "https://login.mql5.com/en/users/Solitonic"
#property version   "1.00"
#include <FRACTAL\CFractalIndexLine.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CFractalSeriesSet
  {
private:
   ENUM_TIMEFRAMES   timeframe; // Step of calculated estimates

   int               ArrayLength;
   CFractalIndexLine FractalIndexLine;

   double            hSampling;    // series discretization step in HOURS

   CFractalIndexLine IndexCalculater;
public:

   int               IndexCount;   // number of calculated local indexes
   double            MuIndexes[];  // Array of local fractal indexes Mu at relevant moments
   double            HURST[];      // Array of Hurst exponent values at relevant moments
   double            R2detSet[];   // Values of Determination Coefficients at relevant moments
   double            KRSet[];      // Values of Mu calculation stability at relevant moments

   CXY_Array        *ConfIntervals; // Confidence intervals for each Mu

                     CFractalSeriesSet();
                    ~CFractalSeriesSet();
   bool              AllocateMemory(int ndat);
   bool              fFractalSegmentSeriesAnalysis(const double &series[],const int idxStart,const int idxStop,const double hourSampling,int windWidth=32,int windShift=1);

  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CFractalSeriesSet::CFractalSeriesSet()
  {
   timeframe=PERIOD_D1; // Default
   ConfIntervals=new CXY_Array();
   ArrayLength = 0;
   IndexCount  = 0;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CFractalSeriesSet::~CFractalSeriesSet()
  {
   ArrayFree(MuIndexes);
   ArrayFree(HURST);
   ArrayFree(R2detSet);
   ArrayFree(KRSet);

   delete ConfIntervals;
  }
//----------------------------------------------------------------------------------------------------------------------------------------------------------------------

bool CFractalSeriesSet::AllocateMemory(int ndat)
  {
   int nn=2*ndat;
   ArrayResize(MuIndexes,nn,ndat);
   ArrayResize(HURST,nn,ndat);
   ArrayResize(R2detSet,nn,ndat);
   nn=ArrayResize(KRSet,nn,ndat);

   ConfIntervals.Resize(nn,ndat);
   return (nn>0)? true:false;
  }
//----------------------------------------------------------------------------------------------------------------------------------------------------------------------

bool CFractalSeriesSet::fFractalSegmentSeriesAnalysis(const double &series[],const int idxStart,const int idxStop,const double hourSampling,
                                                      int windWidth=32,int windShift=1)
  {
   int count,idxLast,CircleCount;

   hSampling=hourSampling;                         // Time of decimation (sampling) of the series in HOURS   

   count   =  idxStop - idxStart+1;                // number of points in the series SAMPLING
   idxLast =  idxStop - windWidth;                 // index for loading the last analyzed fragment 
   CircleCount = 1+(int) floor(idxLast/windShift); // number of calculation cycles for processing a series using segments

   double WRKSer[];
   ArrayResize(WRKSer,count+1); // working array
   for(int i=0; i<count; i++) WRKSer[i]=series[idxStart+i];

   bool curAsSer=ArrayGetAsSeries(series);
   if(!curAsSer) // IF THE INCOMING SERIES IS NOT SORTED "FROM NEW TO OLD", REVERSE THE BLOCK
     {
      ArraySetAsSeries(WRKSer,true); // reversing the series - FIRST NEW, THEN OLD  
     }

   int N0,N1,i,ii;

   for(i=0; i<CircleCount; i++)
     {
      N0=i*windShift;  // boundary indexes which select a fragment in the time series data block
      N1= N0 + windWidth;

      IndexCalculater.CalculateFractalIndex(series,N0,N1,hSampling,windWidth);

      ii=(CircleCount-1-i)+IndexCount;// FRESH ESTIMATES ARE STORED AT THE END OF ARRAYS

      MuIndexes[ii]=IndexCalculater.Mu;
      HURST[ii]=IndexCalculater.Hurst;
      R2detSet[ii]=IndexCalculater.R2det;
      KRSet[ii]=IndexCalculater.KR2;
      ConfIntervals.x[ii]=IndexCalculater.ConfInterval[0]; // BORDERS OF THE CONFIDENCE INTERVAL
      ConfIntervals.y[ii]= IndexCalculater.ConfInterval[1];
     }

   IndexCount+=i;  // save the number of parameter calculations performed

   ArrayFree(WRKSer); // delete the working array

   return (i==CircleCount)?true:false;
  }
//----------------------------------------------------------------------------------------------------------------------------------------------------------------------
